# MWB Sample App

This sample app demonstrates how the MWB pattern could be implemented within a small scope.

The app itself fetches data from [JSON placeholder](https://jsonplaceholder.typicode.com/) and displays it.

## General Architecture

The general approach with the MWB pattern is to separate business logic and data access from UI classes. 
This can benefit the app since you'll be able to test abstract logic separately, 
have a more clean architecture and refrain from mixing code in places where it doesn't really belong.

<p align="center">
  <img width="70%" src="./doc/MWB_page.png">
</p>

In the image above you see how the sample app uses this pattern. A page (analog to an `Activity` or a `ViewController`) accesses the Bloc instance 
via the enveloping `BlocProvider` widget and executes functions inside its bloc member variable.

**NOTE:** In order for the BlocProvider to work properly, it needs to be created **above** the widget-tree scope it envelopes.

To keep things simple for that you can use a static "factory pattern" (but **not** an actual factory function) instead of a widget instance, which would always create a BlocProvider 
and a Bloc instance around your page.
```dart
static Widget create() {
    return BlocProvider(
      bloc: PostsPageBloc(),
      child: PostsPage(),
    );
}
```
That way you'd have less of a hassle to make Bloc creation consistent.

### Example inside app

You can refresh the list of all posts with the button inside the AppBar. This is what happens if you tap it.

1. Set a local variable inside the State of a widget to the instance of a bloc
```dart
@override
  void initState() {
    super.initState();
    _bloc = BlocProvider.of(context);
}
```
2. Execute functions of that Bloc instance where you need to
```dart
IconButton(
    icon: Icon(Icons.refresh),
    onPressed: _bloc.refreshList,
)
```    
3. Render a new list if the posts subject received new data
```dart
StreamBuilder(
    stream: _bloc.postsStream,
    builder: (BuildContext context, AsyncSnapshot<List<Post>> snapshot) {
      if (snapshot.hasData) {
        return ListView(
          children: snapshot.data
              .map((post) => _createPostListItem(post))
              .toList(),
        );
      } else {
        return Center(
          child: Text("No posts yet"),
        );
      }
    },
  ),
```   

**BUT:** It is not always best case to use a `StreamBuiler` for responsive UI. 

This is where the usual Bloc pattern diverges from MWB. 

Sometimes it is simple enough to just use an interface and trigger actions of the UI directly. 

For instance: If you tap a post, the sample app displays more info and comments of that post. If you tap a comment, a SnackBar will display the commenters
email address. This is a one-time action which would be difficult to implement with a `StreamBuilder`. Instead, the sample app uses a simple interface like the MVP
pattern to provide a communication channel,

````dart
class _DetailPageState extends State<DetailPage> implements DetailBlocReceiver{
    @override
    void initState() {
      super.initState();
      _bloc = BlocProvider.of(context);
      _bloc.receiver = this;
    }
    
    @override
    void showMessage(String message) {
      _scaffoldKey.currentState.showSnackBar(SnackBar(
        content: Text(message),
      ));
    }
}
````
which the Bloc instance then uses.
````dart
void showCommentEmail(Comment comment) {
  receiver?.showMessage(comment.email);
}
````

## App architecture

The sample app also shows how you can create a global State for the entire app and provide access to different kind of objects.
This is done with an **AppBloc**. Like an Application class in Android this specific Bloc has only 1 instance for the entire app 
and acts as a holder for all global singletons. Same goes for it's `BlocProvider` widget instance.

<p align="center">
  <img width="60%" src="./doc/MWB_app.png">
</p>

A useful thing to have for more concise access to the `AppBloc` is a mixin.
```dart
mixin AppBlocProvider {
  AppBloc get appBloc => AppBloc._instance;
  Repository get repository => appBloc.repository;
  NavigationService get navService => appBloc.navigationService;
}
```
